@@ -1501,6 +1501,14 @@ function EnzymeDumpModuleRef(mod)
     ccall((:EnzymeDumpModuleRef, libEnzyme), Cvoid, (LLVM.API.LLVMModuleRef,), mod)
 end
 
+function EnzymeDumpValueRef(mod)
+    ccall((:EnzymeDumpValueRef, libEnzyme), Cvoid, (LLVM.API.LLVMValueRef,), mod)
+end
+
+function EnzymeDumpTypeRef(mod)
+    ccall((:EnzymeDumpTypeRef, libEnzyme), Cvoid, (LLVM.API.LLVMValueRef,), mod)
+end
+
 EnzymeComputeByteOffsetOfGEP(B, V, T) = LLVM.Value(
     ccall(
         (:EnzymeComputeByteOffsetOfGEP, libEnzyme),
@@ -1545,8 +1553,10 @@ EnzymeAnonymousAliasScope(dom::LLVM.Metadata, str) = LLVM.Metadata(
 EnzymeFixupJuliaCallingConvention(f) = ccall(
     (:EnzymeFixupJuliaCallingConvention, libEnzyme),
     Cvoid,
-    (LLVM.API.LLVMValueRef,),
+    (LLVM.API.LLVMValueRef, UInt8),
     f,
+    # only on julia before 1.12 did sret contain actual data
+    VERSION < v"1.12"
 )
 EnzymeFixupBatchedJuliaCallingConvention(f) = ccall(
     (:EnzymeFixupBatchedJuliaCallingConvention, libEnzyme),
