@@ -173,6 +173,47 @@ function add_trampoline!(jd, (lljit, lctm, ism), entry, target)
     LLVM.lookup(lljit, entry)
 end
 
+function prepare!(mod)
+    for f in collect(functions(mod))
+        ptr = fix_ptr_lookup(LLVM.name(f))
+        if ptr === nothing
+            continue
+        end
+        ptr = reinterpret(UInt, ptr)
+        ptr = LLVM.ConstantInt(ptr)
+        ptr = LLVM.const_inttoptr(ptr, LLVM.PointerType(LLVM.function_type(f)))
+        replace_uses!(f, ptr)
+        Compiler.eraseInst(mod, f)
+    end
+    for g in collect(globals(mod))
+        if !startswith(LLVM.name(g), "ejl_inserted\$")
+           continue
+        end
+        _, ogname, load1, initaddr = split(LLVM.name(g), "\$")
+
+        load1 = load1 == "true"
+            initaddr = parse(UInt, initaddr)
+        ptr = Base.reinterpret(Ptr{Ptr{Cvoid}}, initaddr)
+        if load1
+           ptr = Base.unsafe_load(ptr, :unordered)
+        end
+                
+        obj = Base.unsafe_pointer_to_objref(ptr)
+	
+        # Let's try a de-bind for 1.10 lux
+        if isa(obj, Core.Binding)
+           ptr = Compiler.unsafe_to_ptr(obj.value)
+        end
+
+        ptr = reinterpret(UInt, ptr)
+        ptr = LLVM.ConstantInt(ptr)
+        ptr = LLVM.const_inttoptr(ptr, LLVM.PointerType(LLVM.StructType(LLVM.LLVMType[])))
+        ptr = LLVM.const_addrspacecast(ptr, LLVM.PointerType(LLVM.StructType(LLVM.LLVMType[]), 10))
+        replace_uses!(g, ptr)
+        Compiler.eraseInst(mod, g)
+    end
+end
+
 function get_trampoline(job)
     compiler = jit[]
     lljit = compiler.jit
@@ -217,6 +258,7 @@ function get_trampoline(job)
                 Compiler.eraseInst(mod, other_func)
             end
 
+	    prepare!(mod)
             tsm = move_to_threadsafe(mod)
 
             il = LLVM.IRCompileLayer(lljit)
@@ -241,17 +283,7 @@ function get_trampoline(job)
 end
 
 function add!(mod)
-    for f in collect(functions(mod))
-        ptr = fix_ptr_lookup(LLVM.name(f))
-        if ptr === nothing
-            continue
-        end
-        ptr = reinterpret(UInt, ptr)
-        ptr = LLVM.ConstantInt(ptr)
-        ptr = LLVM.const_inttoptr(ptr, LLVM.PointerType(LLVM.function_type(f)))
-        replace_uses!(f, ptr)
-        Compiler.eraseInst(mod, f)
-    end
+    prepare!(mod)
     lljit = jit[].jit
     jd = LLVM.JITDylib(lljit)
     tsm = move_to_threadsafe(mod)
