@@ -785,8 +785,8 @@ function Base.showerror(io::IO, ece::EnzymeMutabilityException)
     print(io, "EnzymeMutabilityException: ", msg, '\n')
 end
 
-struct EnzymeRuntimeActivityError{MT,WT} <: EnzymeError
-    msg::Cstring
+struct EnzymeRuntimeActivityError{ST, MT,WT} <: EnzymeError
+    msg::ST
     mi::MT
     world::WT
 end
@@ -831,7 +831,11 @@ function Base.showerror(io::IO, ece::EnzymeRuntimeActivityError)
         )
     end
     println(io)
-    msg = Base.unsafe_string(ece.msg)
+    msg = if ece.msg isa Cstring
+        Base.unsafe_string(ece.msg)
+    else
+        ece.msg
+    end
     print(io, msg, '\n')
 end
 
@@ -990,6 +994,10 @@ function julia_error(
             # Need to convert function to string, since when the error is going to be printed
             # the module might have been destroyed
             ir = string(val)
+        elseif val isa LLVM.GlobalVariable
+            # Need to convert global to string, since when the error is going to be printed
+            # the module might have been destroyed
+            ir = string(val)
         else
             # Need to convert function to string, since when the error is going to be printed
             # the module might have been destroyed
@@ -1064,7 +1072,18 @@ function julia_error(
                 print(io, "Current scope: \n")
                 print(io, ir)
             end
-            if !isa(val, LLVM.Argument)
+	    legal, obj = absint(val)
+	    if legal
+		obj0 = obj
+		obj = unbind(obj)
+	        println(io, "\nValue of type: ", Core.Typeof(obj))
+		println(io ,  " of value    : ", obj)
+		if obj0 isa Core.Binding
+		println(io ,  " binding     : ", obj0)	    
+		end
+		println(io)
+	    end
+	    if !isa(val, LLVM.Argument) && !isa(val, LLVM.GlobalVariable) 
                 print(io, "\n Inverted pointers: \n")
                 ip = API.EnzymeGradientUtilsInvertedPointersToString(gutils)
                 sval = Base.unsafe_string(ip)
@@ -1165,11 +1184,21 @@ function julia_error(
     elseif errtype == API.ET_InternalError
         throw(EnzymeInternalError(msg, ir, bt))
     elseif errtype == API.ET_GCRewrite
-        msgN = sprint() do io::IO
+        data2 = LLVM.Value(data2)
+        fn = LLVM.Function(LLVM.API.LLVMGetParamParent(data2::LLVM.Argument))
+        @static if VERSION < v"1.11"
+	    sretkind = LLVM.kind(if LLVM.version().major >= 12
+		LLVM.TypeAttribute("sret", LLVM.Int32Type())
+	    else
+		LLVM.EnumAttribute("sret")
+	    end)
+	    if occursin("Could not find use of stored value", msg) && length(parameters(fn)) >= 1 && any(LLVM.kind(attr) == sretkind for attr in collect(LLVM.parameter_attributes(fn, 1)))
+		return C_NULL
+	    end
+        end
+	msgN = sprint() do io::IO
             print(io, msg)
             println(io)
-            data2 = LLVM.Value(data2)
-            fn = LLVM.Function(LLVM.API.LLVMGetParamParent(data2::LLVM.Argument))
             println(io, "Fn = ", string(fn))
             println(io, "arg = ", string(data2::LLVM.Argument))
             if data !== C_NULL
@@ -1244,30 +1273,6 @@ function julia_error(
                 return seen[cur]
             end
 
-@static if VERSION < v"1.11-"
-else   
-	if isa(cur, LLVM.LoadInst) && isa(value_type(cur), LLVM.PointerType) && LLVM.addrspace(value_type(operands(cur)[1])) == Derived
-                    larg, off = get_base_and_offset(operands(cur)[1]; inst=ncur, inttoptr=true)
-		    if isa(larg, LLVM.ConstantInt) && off == sizeof(Int)
-			ptr = reinterpret(Ptr{Cvoid}, convert(UInt, larg))
-			obj = Base.unsafe_pointer_to_objref(ptr)
-                        if obj isa Memory && obj == typeof(obj).instance
-                            return make_batched(ncur, prevbb)
-                        end
-		    end
-                end
-	if isa(cur, LLVM.ConstantExpr) && isa(value_type(cur), LLVM.PointerType) && LLVM.addrspace(value_type(cur)) == Derived
-		larg, off = get_base_and_offset(cur; inst=first(instructions(position(prevbb))), inttoptr=true)
-		if isa(larg, LLVM.ConstantInt) && (off == sizeof(Int) || off == 0)
-			ptr = reinterpret(Ptr{Cvoid}, convert(UInt, larg))
-			obj = Base.unsafe_pointer_to_objref(ptr)
-                        if obj isa Memory && obj == typeof(obj).instance
-                            return make_batched(ncur, prevbb)
-                        end
-		    end
-                end
-end
-                
 		if isa(cur, LLVM.LoadInst)
                     larg, off = get_base_and_offset(operands(cur)[1])
 		    if off == 0 && isa(larg, LLVM.AllocaInst)
@@ -1311,11 +1316,15 @@ end
                 end
 
                 legal2, obj = absint(cur)
-
+		obj0 = obj
                 # Only do so for the immediate operand/etc to a phi, since otherwise we will make multiple
                 if legal2
+		   obj = unbind(obj)
+		   if is_memory_instance(obj) || (obj isa Core.SimpleVector && length(obj) == 0)
+			return make_batched(ncur, prevbb)
+		   end
                    if active_reg(TT, world) == ActiveState &&
-                   isa(cur, LLVM.ConstantExpr) &&
+		     ( isa(cur, LLVM.ConstantExpr) || isa(cur, LLVM.GlobalVariable)) &&
                    cur == data2
                     if width == 1
                         if mode == API.DEM_ForwardMode
@@ -1348,12 +1357,6 @@ end
                     end
                     end
 
-@static if VERSION < v"1.11-"
-else    
-                    if obj isa Memory && obj == typeof(obj).instance
-                        return make_batched(ncur, prevbb)
-                    end
-end
                 end
 
 @static if VERSION < v"1.11-"
@@ -1362,7 +1365,8 @@ else
                     larg, off = get_base_and_offset(operands(cur)[1])
                     if isa(larg, LLVM.LoadInst)
                         legal2, obj = absint(larg)
-                        if legal2 && obj isa Memory && obj == typeof(obj).instance
+			obj = unbind(obj)
+			if legal2 && is_memory_instance(obj)
                             return make_batched(ncur, prevbb)
                         end
                     end
@@ -1370,7 +1374,11 @@ else
 end
 
                 badval = if legal2
-                    string(obj) * " of type" * " " * string(TT)
+                    sv = string(obj) * " of type" * " " * string(TT)
+		    if obj0 isa Core.Binding
+			sv = sv *" binded at "*string(obj0)
+		    end
+		    sv
                 else
                     "Unknown object of type" * " " * string(TT)
                 end
@@ -1531,7 +1539,7 @@ end
                 end
             end
            
-            if isa(cur, LLVM.LoadInst) || isa(cur, LLVM.BitCastInst) || isa(cur, LLVM.AddrSpaceCastInst) || (isa(cur, LLVM.GetElementPtrInst) && all(Base.Fix2(isa, LLVM.ConstantInt), operands(cur)[2:end]))
+	    if isa(cur, LLVM.LoadInst) || isa(cur, LLVM.BitCastInst) || isa(cur, LLVM.AddrSpaceCastInst) || (isa(cur, LLVM.GetElementPtrInst) && all(Base.Fix2(isa, LLVM.ConstantInt), operands(cur)[2:end])) || (isa(cur,LLVM.ConstantExpr) &&  opcode(cur) in (LLVM.API.LLVMBitCast, LLVM.API.LLVMAddrSpaceCast, LLVM.API.LLVMGetElementPtr))
                 lhs = make_replacement(operands(cur)[1], prevbb)
                 if illegal
                     return ncur
@@ -1667,9 +1675,9 @@ end
         mode = Enzyme.API.DEM_ReverseModeCombined
 
         if mi !== nothing
-            emit_error(b, nothing, (msg2, mi, world), EnzymeRuntimeActivityError{Core.MethodInstance, UInt})
+            emit_error(b, nothing, (msg2, mi, world), EnzymeRuntimeActivityError{Cstring, Core.MethodInstance, UInt})
         else
-            emit_error(b, nothing, msg2, EnzymeRuntimeActivityError{Nothing, Nothing})
+            emit_error(b, nothing, msg2, EnzymeRuntimeActivityError{Cstring, Nothing, Nothing})
         end
         return C_NULL
     elseif errtype == API.ET_GetIndexError
