@@ -133,8 +133,10 @@ include("parallelrules.jl")
             (
                 "ijl_f__apply_latest",
                 "ijl_f__call_latest",
+                "ijl_f_invokelatest",
                 "jl_f__apply_latest",
                 "jl_f__call_latest",
+                "jl_f_invokelatest",
             ),
         )
             return common_apply_latest_fwd(2, B, orig, gutils, normalR, shadowR)
@@ -165,11 +167,25 @@ include("parallelrules.jl")
         end
     end
 
-    err = emit_error(
-        B,
-        orig,
-        "Enzyme: jl_call calling convention not implemented in forward for " * string(orig),
-    )
+    pf = LLVM.parent(LLVM.parent(orig))::LLVM.Function
+    mi, _ = enzyme_custom_extract_mi(pf, false) #=error=#
+    world = enzyme_extract_world(pf)
+
+    if mi !== nothing
+        err = emit_error(
+            B,
+            orig,
+            ("Enzyme: jl_call calling convention not implemented in forward for " * string(orig), mi, world),
+            EnzymeRuntimeExceptionMI
+        )
+    else
+        err = emit_error(
+            B,
+            orig,
+            "Enzyme: jl_call calling convention not implemented in forward for " * string(orig),
+            EnzymeRuntimeException
+        )
+    end
 
     newo = new_from_original(gutils, orig)
 
@@ -212,8 +228,10 @@ end
             (
                 "ijl_f__apply_latest",
                 "ijl_f__call_latest",
+                "ijl_f_invokelatest",
                 "jl_f__apply_latest",
                 "jl_f__call_latest",
+                "jl_f_invokelatest",
             ),
         )
             return common_apply_latest_augfwd(2, B, orig, gutils, normalR, shadowR, tapeR)
@@ -244,12 +262,26 @@ end
         end
     end
 
-    err = emit_error(
-        B,
-        orig,
-        "Enzyme: jl_call calling convention not implemented in aug_forward for " *
-        string(orig),
-    )
+    pf = LLVM.parent(LLVM.parent(orig))::LLVM.Function
+    mi, _ = enzyme_custom_extract_mi(pf, false) #=error=#
+    world = enzyme_extract_world(pf)
+
+    if mi !== nothing
+        err = emit_error(
+            B,
+            orig,
+            ("Enzyme: jl_call calling convention not implemented in aug_forward for " * string(orig), mi, world),
+            EnzymeRuntimeExceptionMI
+        )
+    else
+        err = emit_error(
+            B,
+            orig,
+            "Enzyme: jl_call calling convention not implemented in aug_forward for " * string(orig),
+            EnzymeRuntimeException
+        )
+    end
+
     newo = new_from_original(gutils, orig)
 
     API.moveBefore(newo, err, B)
@@ -290,8 +322,10 @@ end
             (
                 "ijl_f__apply_latest",
                 "ijl_f__call_latest",
+                "ijl_f_invokelatest",
                 "jl_f__apply_latest",
                 "jl_f__call_latest",
+                "jl_f_invokelatest",
             ),
         )
             common_apply_latest_rev(2, B, orig, gutils, tape)
@@ -330,11 +364,25 @@ end
         end
     end
 
-    emit_error(
-        B,
-        orig,
-        "Enzyme: jl_call calling convention not implemented in reverse for " * string(orig),
-    )
+    pf = LLVM.parent(LLVM.parent(orig))::LLVM.Function
+    mi, _ = enzyme_custom_extract_mi(pf, false) #=error=#
+    world = enzyme_extract_world(pf)
+
+    if mi !== nothing
+        err = emit_error(
+            B,
+            orig,
+            ("Enzyme: jl_call calling convention not implemented in reverse for " * string(orig), mi, world),
+            EnzymeRuntimeExceptionMI
+        )
+    else
+        err = emit_error(
+            B,
+            orig,
+            "Enzyme: jl_call calling convention not implemented in reverse for " * string(orig),
+            EnzymeRuntimeException
+        )
+    end
 
     return nothing
 end
@@ -1528,9 +1576,6 @@ end
         offset = new_from_original(gutils, origops[2])
         offset = lookup_value(gutils, offset, B)
 
-        currentBlock = Base.position(B)
-        ogname = LLVM.name(currentBlock)
-
         fval = if get_runtime_activity(gutils)
             lookup_value(gutils, new_from_original(gutils, origops[1]), B)
         else
@@ -1545,12 +1590,15 @@ end
                 offset
             ]
             if width > 1
-                extract_value!(B, shadowin, idx - 1)
+                args[1] = extract_value!(B, args[1], idx - 1)
             end
 
-            if get_runtime_activity(gutils)
+            if get_runtime_activity(gutils) && endB === nothing
                 cond = icmp!(B, LLVM.API.LLVMIntNE, fval, args[1])
 
+	        currentBlock = Base.position(B)
+        	ogname = LLVM.name(currentBlock)
+
                 nextB = add_reverse_block!(gutils, currentBlock, ogname*"_active")
 
                 endB = add_reverse_block!(gutils, nextB, ogname*"_end", true, false)
@@ -1561,13 +1609,13 @@ end
             end
 
             LLVM.call!(B, fty, delF, args)
+        end
 
-            if get_runtime_activity(gutils)
-                br!(B, endB)
-                set_reverse_block!(gutils, endB)
-                position!(B, endB)
-                currentBlock = endB
-            end
+        if endB !== nothing
+            br!(B, endB)
+            set_reverse_block!(gutils, endB)
+            position!(B, endB)
+            currentBlock = endB
         end
     end
     return nothing
@@ -1602,9 +1650,6 @@ end
 
         i8 = LLVM.IntType(8)
 
-        currentBlock = Base.position(B)
-        ogname = LLVM.name(currentBlock)
-
         fval = if get_runtime_activity(gutils)
             lookup_value(gutils, new_from_original(gutils, origops[1]), B)
         else
@@ -1622,6 +1667,9 @@ end
 
             if get_runtime_activity(gutils)
                 cond = icmp!(B, LLVM.API.LLVMIntNE, fval, anti)
+        
+		currentBlock = Base.position(B)
+		ogname = LLVM.name(currentBlock)
 
                 nextB = add_reverse_block!(gutils, currentBlock, ogname*"_active")
 
@@ -1771,11 +1819,11 @@ end
     end
 
     args = LLVM.Value[]
-    for a in origops[1:end-2]
+    for a in origops[1:4]
         v = invert_pointer(gutils, a, B)
         push!(args, v)
     end
-    push!(args, new_from_original(gutils, origops[end-1]))
+    push!(args, new_from_original(gutils, origops[5]))
     valTys = API.CValueType[
         API.VT_Shadow,
         API.VT_Shadow,
@@ -1905,11 +1953,12 @@ end
     API.moveBefore(newo, err, B)
 
     if unsafe_load(shadowR) != C_NULL
-        valTys = API.CValueType[API.VT_Primal, API.VT_Primal]
-        args = [
-            new_from_original(gutils, operands(orig)[1]),
-            new_from_original(gutils, operands(orig)[2]),
-        ]
+	valTys = API.CValueType[]
+	args = LLVM.Value[]
+	for i in 1:(length(operands(orig))-1)    
+	    push!(valTys, API.VT_Primal)
+	    push!(args, new_from_original(gutils, operands(orig)[i]))
+	end
         normal = call_samefunc_with_inverted_bundles!(B, gutils, orig, args, valTys, false) #=lookup=#
         width = get_width(gutils)
         if width == 1
@@ -1939,12 +1988,13 @@ end
     newo = new_from_original(gutils, orig)
     API.moveBefore(newo, err, B)
     if unsafe_load(shadowR) != C_NULL
-        valTys = API.CValueType[API.VT_Primal, API.VT_Primal]
-        args = [
-            new_from_original(gutils, operands(orig)[1]),
-            new_from_original(gutils, operands(orig)[2]),
-        ]
-        normal = call_samefunc_with_inverted_bundles!(B, gutils, orig, args, valTys, false) #=lookup=#
+	valTys = API.CValueType[]
+	args = LLVM.Value[]
+	for i in 1:(length(operands(orig))-1)    
+	    push!(valTys, API.VT_Primal)
+	    push!(args, new_from_original(gutils, operands(orig)[i]))
+	end
+	normal = call_samefunc_with_inverted_bundles!(B, gutils, orig, args, valTys, false) #=lookup=#
         width = get_width(gutils)
         if width == 1
             shadowres = normal
@@ -2205,7 +2255,7 @@ end
         @fwdfunc(invoke_fwd),
     )
     register_handler!(
-        ("jl_f__apply_latest", "jl_f__call_latest"),
+        ("jl_f__apply_latest", "jl_f__call_latest", "jl_f_invokelatest"),
         @augfunc(apply_latest_augfwd),
         @revfunc(apply_latest_rev),
         @fwdfunc(apply_latest_fwd),
@@ -2343,7 +2393,9 @@ end
         @fwdfunc(new_structt_fwd),
     )
     register_handler!(
-        ("jl_get_binding_or_error", "ijl_get_binding_or_error"),
+        ("jl_get_binding_or_error", "ijl_get_binding_or_error",
+	 "jl_get_binding_value_seqcst", "ijl_get_binding_value_seqcst",
+	 ),
         @augfunc(get_binding_or_error_augfwd),
         @revfunc(get_binding_or_error_rev),
         @fwdfunc(get_binding_or_error_fwd),
